<?php

/**
 * @package Popular Topics
 * @version 1.1
 * @author Diego Andrés <diegoandres_cortes@outlook.com>
 * @copyright Copyright (c) 2015, Diego Andrés
 * @license http://www.mozilla.org/MPL/MPL-1.1.html
 */

if (!defined('SMF'))
	die('No direct access...');

require_once($sourcedir . '/Ohara.php');

class PopularTopics extends Ohara
{
	protected static $className = __CLASS__;
	public $version = '1.1';
	protected $replies;
	protected $views;
	protected $popularity = array();
	protected $hooks = array();

	public function __construct()
	{
		global $board, $topic;

		$this->hooks = array(
			'integrate_menu_buttons' => 'credits',
		);
		
		if (((empty($_REQUEST['action']) || !isset($_REQUEST['action'])) && !empty($board) && empty($topic)) && $this->setting('enable'))
		{
			//Limits
			$this->popularity = $this->GetLimits();
			//Replies and views
			$this->replies = $this->popularity['replies'];
			$this->views = $this->popularity['views'];
			if($this->replies == 0)
				$this->replies = 1;
			if($this->views == 0)
				$this->views = 1;
		}
		
		parent::__construct();
	}

	/**
	 * PopularTopics::GetLimits()
	 *
	 * We steal some queries from the Stats page to use them here.
	 * @global $smcFunc, $modSettings;
	 * @return
	 * @author Diego Andrés <diegoandres_cortes@outlook.com>
	 */
	public function GetLimits()
	{
		global $smcFunc, $modSettings;

		if (($this->popularity = cache_get_data(self::$className, 120)) == null)
		{
			// Topic replies top 10.
			$request = $smcFunc['db_query']('', '
				SELECT MAX(t.num_views) AS num_views, MAX(t.num_replies) AS num_replies
				FROM {db_prefix}topics AS t
				INNER JOIN {db_prefix}boards AS b ON (b.id_board = t.id_board' . (!empty($modSettings['recycle_enable']) && $modSettings['recycle_board'] > 0 ? '
					AND b.id_board != {int:recycle_board}' : '') . ')
				WHERE {query_see_board}' . ($modSettings['postmod_active'] ? '
					AND t.approved = {int:is_approved}' : ''),
				array(
					'recycle_board' => $modSettings['recycle_board'],
					'is_approved' => 1,
				)
			);
			$maximum = $smcFunc['db_fetch_assoc']($request);

			// Get our results
			$this->popularity = array(
				'views' => $maximum['num_views'],
				'replies' => $maximum['num_replies']
			);
			$smcFunc['db_free_result']($request);

			cache_put_data(self::$className, $this->popularity, 120);
		}

		return $this->popularity;
	}

	/**
	 * PopularTopics::formatBar()
	 *
	 * It will make almost everything....
	 * @param int $views, int $replies
	 * @global $context, $modSettings
	 * @return
	 * @author Diego Andrés <diegoandres_cortes@outlook.com>
	 */
	public function formatBar($views, $replies)
	{
		global $context;

		if ($this->setting('enable'))
		{
			// Take care of wrong values in views
			if ($this->setting('views_percent') < 0 || $this->setting('views_percent') > 100)
				$context['PopularTopics_total']['views_percent'] = 70;
			else
				$context['PopularTopics_total']['views_percent'] = $this->setting('views_percent');

			// Take care of wrong values in replies
			if ($this->setting('replies_percent') < 0 || $this->setting('replies_percent') > 100)
				$context['PopularTopics_total']['replies_percent'] = 30;
			else
				$context['PopularTopics_total']['replies_percent'] = $this->setting('replies_percent');

			// Again, use defaults if the sume is more than 100, or even worse, less than 100!
			$total = $context['PopularTopics_total']['replies_percent'] + $context['PopularTopics_total']['views_percent'];
			if ($total > 100 || $total < 100)
			{
				$context['PopularTopics_total']['views_percent'] = 70;
				$context['PopularTopics_total']['replies_percent'] = 30;
			}

			$context['PopularTopics_percent']['views'] = (($views*$context['PopularTopics_total']['views_percent'])/$this->views);
			$context['PopularTopics_percent']['replies'] = (($replies*$context['PopularTopics_total']['replies_percent'])/$this->replies);
			$context['PopularTopics_percent']['total'] = ($context['PopularTopics_percent']['views'] + $context['PopularTopics_percent']['replies']);

			// Colorize the bar
			if (($context['PopularTopics_percent']['total'] > 0 || $context['PopularTopics_percent']['total'] == 0) && $context['PopularTopics_percent']['total'] < 20)
				$context['PopularTopics_percent']['color'] = '#DF0101';
			elseif ($context['PopularTopics_percent']['total'] >= 20 && $context['PopularTopics_percent']['total'] < 40)
				$context['PopularTopics_percent']['color'] = '#DF7401';
			elseif ($context['PopularTopics_percent']['total'] >= 40 && $context['PopularTopics_percent']['total'] < 60)
				$context['PopularTopics_percent']['color'] = '#DBA901';
			elseif ($context['PopularTopics_percent']['total'] >= 60 && $context['PopularTopics_percent']['total'] < 80)
				$context['PopularTopics_percent']['color'] = '#AEB404';
			elseif ($context['PopularTopics_percent']['total'] >= 80)
				$context['PopularTopics_percent']['color'] = '#298A08';

			// Send back an array with the percent and the color
			$context['PopularTopics_percent']['values'] = array($context['PopularTopics_percent']['total'], $context['PopularTopics_percent']['color']);

			return $context['PopularTopics_percent']['values'];

		}

	}

	public function credits(&$crappy)
	{
		global $context;

		if (!empty($context['current_action']) && $context['current_action'] == 'credits')
			$context['copyrights']['mods'][] = '<a href="http://smftricks.com" title="SMF Themes & Mods">Popular Topics &copy Diego Andr&eacute;s & SMF Tricks</a>';
	}
}